@@ -404,7 +404,8 @@ _g_usb_device_save(GUsbDevice *self, JsonBuilder *json_builder, GError **error)
 	/* array of BOS descriptors */
 	bos_descriptors = g_usb_device_get_bos_descriptors(self, &error_bos);
 	if (bos_descriptors == NULL) {
-		g_debug("%s", error_bos->message);
+		if (_g_usb_context_has_flag(priv->context, G_USB_CONTEXT_FLAGS_DEBUG))
+			g_debug("%s", error_bos->message);
 	} else if (bos_descriptors->len > 0) {
 		json_builder_set_member_name(json_builder, "UsbBosDescriptors");
 		json_builder_begin_array(json_builder);
@@ -419,7 +420,8 @@ _g_usb_device_save(GUsbDevice *self, JsonBuilder *json_builder, GError **error)
 	/* array of interfaces */
 	interfaces = g_usb_device_get_interfaces(self, &error_interfaces);
 	if (interfaces == NULL) {
-		g_debug("%s", error_interfaces->message);
+		if (_g_usb_context_has_flag(priv->context, G_USB_CONTEXT_FLAGS_DEBUG))
+			g_debug("%s", error_interfaces->message);
 	} else if (interfaces->len > 0) {
 		json_builder_set_member_name(json_builder, "UsbInterfaces");
 		json_builder_begin_array(json_builder);
@@ -448,6 +450,24 @@ _g_usb_device_save(GUsbDevice *self, JsonBuilder *json_builder, GError **error)
 	return TRUE;
 }
 
+/**
+ * g_usb_device_get_tags:
+ * @self: a #GUsbDevice
+ *
+ * Gets all the tags.
+ *
+ * Returns: (transfer container) (element-type utf8): string tags
+ *
+ * Since: 0.4.4
+ **/
+GPtrArray *
+g_usb_device_get_tags(GUsbDevice *self)
+{
+	GUsbDevicePrivate *priv = GET_PRIVATE(self);
+	g_return_val_if_fail(G_USB_IS_DEVICE(self), NULL);
+	return g_ptr_array_ref(priv->tags);
+}
+
 /**
  * g_usb_device_has_tag:
  * @self: a #GUsbDevice
@@ -501,6 +521,32 @@ g_usb_device_add_tag(GUsbDevice *self, const gchar *tag)
 	g_ptr_array_add(priv->tags, g_strdup(tag));
 }
 
+/**
+ * g_usb_device_remove_tag:
+ * @self: a #GUsbDevice
+ * @tag: a tag, for example `bootloader` or `runtime-reload`
+ *
+ * Removes a tag, which is included in the JSON log to identify the specific device.
+ *
+ * Since: 0.4.4
+ **/
+void
+g_usb_device_remove_tag(GUsbDevice *self, const gchar *tag)
+{
+	GUsbDevicePrivate *priv = GET_PRIVATE(self);
+
+	g_return_if_fail(G_USB_IS_DEVICE(self));
+	g_return_if_fail(tag != NULL);
+
+	for (guint i = 0; i < priv->tags->len; i++) {
+		const gchar *tag_tmp = g_ptr_array_index(priv->tags, i);
+		if (g_strcmp0(tag_tmp, tag) == 0) {
+			g_ptr_array_remove_index(priv->tags, i);
+			return;
+		}
+	}
+}
+
 /* not defined in FreeBSD */
 #ifndef HAVE_LIBUSB_GET_PARENT
 static libusb_device *
@@ -615,6 +661,23 @@ _g_usb_device_get_device(GUsbDevice *self)
 	return priv->device;
 }
 
+/**
+ * g_usb_device_is_emulated:
+ * @self: a #GUsbDevice instance
+ *
+ * Gets if the device is emulated.
+ *
+ * Return value: %TRUE if the device is emulated and not backed by a physical device.
+ *
+ * Since: 0.4.4
+ **/
+gboolean
+g_usb_device_is_emulated(GUsbDevice *self)
+{
+	GUsbDevicePrivate *priv = GET_PRIVATE(self);
+	return priv->device == NULL;
+}
+
 static gboolean
 g_usb_device_libusb_error_to_gerror(GUsbDevice *self, gint rc, GError **error)
 {
@@ -768,7 +831,8 @@ g_usb_device_load_event(GUsbDevice *self, const gchar *id)
 	for (guint i = priv->event_idx; i < priv->events->len; i++) {
 		GUsbDeviceEvent *event = g_ptr_array_index(priv->events, i);
 		if (g_strcmp0(g_usb_device_event_get_id(event), id) == 0) {
-			g_debug("found in-order %s at position %u", id, i);
+			if (_g_usb_context_has_flag(priv->context, G_USB_CONTEXT_FLAGS_DEBUG))
+				g_debug("found in-order %s at position %u", id, i);
 			priv->event_idx = i + 1;
 			return event;
 		}
@@ -778,7 +842,8 @@ g_usb_device_load_event(GUsbDevice *self, const gchar *id)
 	for (guint i = 0; i < priv->events->len; i++) {
 		GUsbDeviceEvent *event = g_ptr_array_index(priv->events, i);
 		if (g_strcmp0(g_usb_device_event_get_id(event), id) == 0) {
-			g_debug("found out-of-order %s at position %u", id, i);
+			if (_g_usb_context_has_flag(priv->context, G_USB_CONTEXT_FLAGS_DEBUG))
+				g_debug("found out-of-order %s at position %u", id, i);
 			return event;
 		}
 	}
@@ -1038,6 +1103,22 @@ g_usb_device_get_events(GUsbDevice *self)
 	return g_ptr_array_ref(priv->events);
 }
 
+/**
+ * g_usb_device_clear_events:
+ * @self: a #GUsbDevice
+ *
+ * Clear all the events saved by the device.
+ *
+ * Since: 0.4.4
+ **/
+void
+g_usb_device_clear_events(GUsbDevice *self)
+{
+	GUsbDevicePrivate *priv = GET_PRIVATE(self);
+	g_return_if_fail(G_USB_IS_DEVICE(self));
+	g_ptr_array_set_size(priv->events, 0);
+}
+
 /**
  * g_usb_device_invalidate:
  * @self: a #GUsbDevice
