@@ -40,7 +40,8 @@ g_usb_interface_finalize(GObject *object)
 	GUsbInterface *self = G_USB_INTERFACE(object);
 
 	g_bytes_unref(self->extra);
-	g_ptr_array_unref(self->endpoints);
+	if (self->endpoints != NULL)
+		g_ptr_array_unref(self->endpoints);
 
 	G_OBJECT_CLASS(g_usb_interface_parent_class)->finalize(object);
 }
@@ -85,6 +86,20 @@ _g_usb_interface_load(GUsbInterface *self, JsonObject *json_object, GError **err
 	self->iface.iInterface =
 	    json_object_get_int_member_with_default(json_object, "Interface", 0x0);
 
+	/* array of endpoints */
+	if (json_object_has_member(json_object, "UsbEndpoints")) {
+		JsonArray *json_array = json_object_get_array_member(json_object, "UsbEndpoints");
+		self->endpoints = g_ptr_array_new_with_free_func(g_object_unref);
+		for (guint i = 0; i < json_array_get_length(json_array); i++) {
+			JsonNode *node_tmp = json_array_get_element(json_array, i);
+			JsonObject *obj_tmp = json_node_get_object(node_tmp);
+			g_autoptr(GUsbEndpoint) endpoint = g_object_new(G_USB_TYPE_ENDPOINT, NULL);
+			if (!_g_usb_endpoint_load(endpoint, obj_tmp, error))
+				return FALSE;
+			g_ptr_array_add(self->endpoints, g_object_ref(endpoint));
+		}
+	}
+
 	/* extra data */
 	str = json_object_get_string_member_with_default(json_object, "ExtraData", NULL);
 	if (str != NULL) {
@@ -151,7 +166,7 @@ _g_usb_interface_save(GUsbInterface *self, JsonBuilder *json_builder, GError **e
 	}
 
 	/* array of endpoints */
-	if (self->endpoints->len > 0) {
+	if (self->endpoints != NULL && self->endpoints->len > 0) {
 		json_builder_set_member_name(json_builder, "UsbEndpoints");
 		json_builder_begin_array(json_builder);
 		for (guint i = 0; i < self->endpoints->len; i++) {
